/* Copyright (C) 2007 Laurent Perez <laurent.perez@unicaen.fr> */
/* Released under the terms of the GNU GPL (version 2 or later). */

#define HAVE_LONG_LONG 1
/* * This file was generated by SLIRP, the (Sl)ang (I)nte(r)face (P)ackage,
 * a S-Lang module code generator for C, C++, and FORTRAN.
 *
 * Copyright (C) 2003-2006 Massachusetts Institute of Technology 
 * Copyright (C) 2002 Michael S. Noble <mnoble@space.mit.edu>
 *
 * SLIRP is free software, and may be used under the conditions stipulated
 * in the COPYRIGHT agreement bundled within the SLIRP distribution.
 */
#include <stdlib.h>		/* SLIRP common {{{ */
#include <unistd.h>
#include <string.h>
#include <stdarg.h>
#include <slang.h>

#include <gtk/gtk.h>
#include <gtkdatabox.h>
#include <gtkdatabox_bars.h>
#include <gtkdatabox_grid.h>
#include <gtkdatabox_lines.h>
#include <gtkdatabox_points.h>
#include <gtkdatabox_cross_simple.h>
#include <gtkdatabox_scale.h>
#include "module.h"

#define V				SLANG_VOID_TYPE
#define SLEU				SL_USAGE_ERROR
#define SLEI				SL_INTRINSIC_ERROR
#define SAFE_DEREF_OPAQUE(po)		(po == NULL ? NULL : po->instance)
#define SAFE_DEREF_ARRAY(pa)		(pa == NULL ? NULL : pa->data)
#define SLang_pop_array(x)		SLang_pop_array(x, 1)
#define SLang_pop_string(x)             SLang_pop_slstring(x)
#define SLang_push_size_t		SLang_push_int
#define SLang_push_ptrdiff_t		SLang_push_int

#define pop_defaultable(argno, kind, type, obj, value) \
   (SLang_Num_Function_Args >= argno ? SLang_pop_##kind ( (type*)&obj) : ((obj = value) || ( (void*)&obj > (void*)0) ) )

#ifdef __cplusplus
#define LINKAGE "C"
#else
#define LINKAGE
#endif

static void Slirp_usage(int i, int j, int flags);

static int slang_abi_mismatch(void)
{
   long module_abi = SLANG_VERSION / 10000;
   long app_abi    = SLang_Version / 10000;

   if (module_abi != app_abi) {
	SLang_verror(SL_APPLICATION_ERROR, (char*)
		"S-Lang library abi mismatch\nmodule: %s, application: %s",
		SLANG_VERSION_STRING, SLang_Version_String);
	return 1;
   }
			        
   return 0;
} /* }}} */

#if SLANG_VERSION < 20000 /* {{{ */
#define SLclass_get_class_id(x)	(x)->cl_data_type
#define SLang_get_error()	SLang_Error
#define SLang_set_error(x)	(SLang_Error = (x))
#define SLang_pop_double(x)	SLang_pop_double(x, NULL, NULL)
#define SLang_pop_uint		SLang_pop_uinteger
#define SLang_pop_int		SLang_pop_integer
#define SLang_push_uint		SLang_push_uinteger
#define SLang_push_int		SLang_push_integer
#define SLutf8_enable(x)	x
#define SLang_pop_struct	_SLang_pop_struct
#define SLang_free_struct	_SLstruct_delete_struct
#define SLstack_depth		_SLstack_depth
#define SLuindex_Type		unsigned int
#define SLindex_Type		int
extern LINKAGE int _SLstack_depth(void);
#endif /* }}} */

#define BEGIN_DECLS
#define END_DECLS
extern SLtype GtkOpaque_Type;
extern SLtype GObject_Type;
extern SLtype GdkDrawable_Type;
extern SLtype GdkPixmap_Type;
extern SLtype GdkBitmap_Type;
extern SLtype GdkGC_Type;
extern SLtype GdkPixbuf_Type;
extern SLtype GdkPixbufAnimation_Type;
extern SLtype GtkObject_Type;
extern SLtype GtkCellRenderer_Type;
extern SLtype GtkCellRendererPixbuf_Type;
extern SLtype GtkCellRendererText_Type;
extern SLtype GtkCellRendererToggle_Type;
extern SLtype GtkItemFactory_Type;
extern SLtype GtkTreeViewColumn_Type;
extern SLtype GtkTooltips_Type;
extern SLtype GtkAdjustment_Type;
extern SLtype GtkWidget_Type;
extern SLtype GdkCursor_Type;
extern SLtype GdkRegion_Type;
extern SLtype GtkIconSource_Type;
extern SLtype GtkIconSet_Type;
extern SLtype GtkTreeIter_Type;
extern SLtype GtkTextIter_Type;
extern SLtype GtkTreePath_Type;
#define SLIRP_EXTERN	extern
BEGIN_DECLS				/* Opaque handling code {{{ */

#define SLIRP_ABI_VERSION			10500

typedef void            (*FINALIZER)            (void*);
typedef void            (*INITIALIZER)          (void*);

typedef struct _Slirp_Type {
   SLang_Class_Type	*slclass;	/* SLang type class for this type */
   struct _Slirp_Type	*parent;	/* NULL if type has no ancestors  */
   FINALIZER		finalizer;
   INITIALIZER		initializer;
} Slirp_Type;

typedef struct _Slirp_Opaque {
   Slirp_Type *type;
   SLang_MMT_Type *mmt;
   void* instance;
   unsigned int owns_ref;
} Slirp_Opaque;
#define SLang_free_opaque(o)	if (o != NULL) SLang_free_mmt(o->mmt)

static int opaque_typecast (SLtype from_type, VOID_STAR from_p,
			unsigned int num, SLtype to_type, VOID_STAR to_p)
{
   unsigned int i;
   SLang_MMT_Type **from = (SLang_MMT_Type**)from_p;
   SLang_MMT_Type **to   = (SLang_MMT_Type**)to_p;

   (void) from_type; (void) to_type; 

   for (i=0; i < num; i++) {
	to[i] = from[i];	/* FIXME: typeof() still reports prev type */
	if (from[i] != NULL)
	   SLang_inc_mmt(from[i]);
   }

   return 1;
}

static int opaque_search(const void* key, const void *elem)
{
   return *(SLtype*)key - SLclass_get_class_id((*(Slirp_Type**)elem)->slclass);
}

static void destroy_opaque(SLtype type, VOID_STAR w)
{
   Slirp_Opaque *ot = (Slirp_Opaque*)w;

   (void) type;

   if (ot->type->finalizer != NULL)
      (*ot->type->finalizer) (ot->instance);

   SLfree( (char*)w );
}
END_DECLS

SLIRP_EXTERN SLtype void_ptr_Type;	/* These opaque pointer types must */
SLIRP_EXTERN SLtype string_ptr_Type;	/* be kept in sync with opaques.c  */
SLIRP_EXTERN SLtype uchar_ptr_Type;
SLIRP_EXTERN SLtype short_ptr_Type;
SLIRP_EXTERN SLtype ushort_ptr_Type;
SLIRP_EXTERN SLtype int_ptr_Type;
SLIRP_EXTERN SLtype uint_ptr_Type;
SLIRP_EXTERN SLtype long_ptr_Type;
SLIRP_EXTERN SLtype ulong_ptr_Type;
SLIRP_EXTERN SLtype float_ptr_Type;
SLIRP_EXTERN SLtype double_ptr_Type;
SLIRP_EXTERN SLtype opaque_ptr_Type;
SLIRP_EXTERN SLtype file_ptr_Type;

typedef struct _Reserved_Opaque_Type {
   char	  *name;
   SLtype *type;		/* SLtype of opaque pointer defined by this */
   SLtype masked_type;		/* SLtype of datum to which this points */
} Reserved_Opaque_Type;

SLIRP_EXTERN Slirp_Type	**Slirp_Opaque_Types;
SLIRP_EXTERN SLtype 	Slirp_Num_Opaque_Types;
SLIRP_EXTERN SLtype	First_Opaque_Type;
SLIRP_EXTERN SLtype	Last_Reserved_Opaque_Type;

static Reserved_Opaque_Type Reserved_Opaque_Types[] = {

   { (char*)"void_ptr",		&void_ptr_Type,		SLANG_ANY_TYPE,    },
   { (char*)"int_ptr",		&int_ptr_Type,		SLANG_INT_TYPE,    },
   { (char*)"double_ptr",	&double_ptr_Type,	SLANG_DOUBLE_TYPE, },
   { (char*)"opaque_ptr",	&opaque_ptr_Type,	SLANG_ANY_TYPE,	   },
   { (char*)"file_ptr",		&file_ptr_Type,		SLANG_FILE_PTR_TYPE,},
   { (char*)"float_ptr",	&float_ptr_Type,	SLANG_FLOAT_TYPE,  },
   { (char*)"long_ptr",		&long_ptr_Type,		SLANG_LONG_TYPE,   },
   { (char*)"string_ptr",	&string_ptr_Type,	SLANG_STRING_TYPE, },
   { (char*)"uint_ptr",		&uint_ptr_Type,		SLANG_UINT_TYPE,   },
   { (char*)"short_ptr",	&short_ptr_Type,	SLANG_SHORT_TYPE,  },
   { (char*)"ulong_ptr",	&ulong_ptr_Type,	SLANG_ULONG_TYPE,  },
   { (char*)"ushort_ptr",	&ushort_ptr_Type,	SLANG_USHORT_TYPE, },
   { (char*)"uchar_ptr",	&uchar_ptr_Type,	SLANG_UCHAR_TYPE,  },
   { NULL, NULL, 0, }
};

#define NUM_RESERVED_OPAQUES \
	sizeof(Reserved_Opaque_Types) / sizeof(Reserved_Opaque_Type) - 1

static Slirp_Type* sltype_to_slirp_type(SLtype sltype)
{
   Slirp_Type **ot = (Slirp_Type**) bsearch( (const void*) &sltype,
	 		(const void*) Slirp_Opaque_Types,
			(size_t) Slirp_Num_Opaque_Types,
			sizeof(Slirp_Type*), opaque_search);

   if (ot != NULL) return *ot;
   return NULL;
}

static SLang_MMT_Type*
create_opaque_mmt(SLtype type, void *instance, unsigned int owns_ref)
{
   SLang_MMT_Type *mmt = NULL;
   Slirp_Opaque *ot = (Slirp_Opaque*) SLmalloc(sizeof(Slirp_Opaque));

   if (ot != NULL) {

	ot->instance = instance;
	ot->owns_ref = owns_ref;
	ot->mmt      = NULL;

	if ( (ot->type = sltype_to_slirp_type(type)) != NULL) {

	   if (ot->type->initializer != NULL)
		(*ot->type->initializer) (ot->instance);

	   mmt = SLang_create_mmt (type, (VOID_STAR) ot);
	}

	if (mmt == NULL)
	   SLfree((char*)ot);
   }

   return mmt;
}

static int SLang_push_opaque(SLtype type, void *instance, unsigned int owns_ref)
{
   SLang_MMT_Type *mmt;

   if (instance == NULL)
      return SLang_push_null();

   mmt = create_opaque_mmt(type, instance, owns_ref);
   if (NULL == mmt)
      return -1;

   return SLang_push_mmt(mmt);
}

static int SLang_pop_opaque(SLtype type, void **instance, Slirp_Opaque **o)
{
   Slirp_Opaque *ot;
   SLang_MMT_Type *mmt = NULL;

   if (type == file_ptr_Type && SLang_peek_at_stack() == SLANG_FILE_PTR_TYPE) {
	FILE *fp;
	if (SLang_pop_fileptr(&mmt, &fp) == 0) {
	   ot = (Slirp_Opaque*) SLmalloc(sizeof(Slirp_Opaque));
	   if (ot == NULL) { SLang_free_mmt(mmt); return -1; }
	   ot->mmt = mmt; *instance = (void*)fp; *o = ot; return 0;
	}
   }
   else if (	(mmt = SLang_pop_mmt(type)) != NULL &&
		(ot = (Slirp_Opaque*) SLang_object_from_mmt (mmt)) != NULL &&
		ot->instance != NULL) {

	ot->mmt = mmt;
	if (instance != NULL)
	   *instance = ot->instance;
	*o = ot;
	return 0;
   }

   if (instance != NULL) *instance = NULL;
   SLang_free_mmt (mmt);
   return -1;
}

static unsigned int
allocate_opaque(char *name, FINALIZER fzer, INITIALIZER izer, SLtype parent_id)
{
   /* Returning from here with anything but a positive id is fatal */
   SLtype new_type, ancestor_type;
   Slirp_Type *type , *parent = NULL;
   SLang_Class_Type *new_class = SLclass_allocate_class (name);

   if (new_class == NULL) return SLANG_UNDEFINED_TYPE;

   (void) SLclass_set_destroy_function (new_class, destroy_opaque);
   if (-1 == SLclass_register_class (new_class, SLANG_VOID_TYPE,
				     sizeof(Slirp_Opaque),
				     SLANG_CLASS_TYPE_MMT))
	return SLANG_UNDEFINED_TYPE;

   new_type = SLclass_get_class_id(new_class);
   if (parent_id) {

	type = parent = sltype_to_slirp_type(parent_id);

	while (type) {

	   ancestor_type = SLclass_get_class_id(type->slclass);

	   /* Support downcasting ancestor types to this type */
	   if (-1 == SLclass_add_typecast (ancestor_type, new_type,
		    					opaque_typecast, 1))
		return SLANG_UNDEFINED_TYPE;

	   /* Support upcasting this type to each ancestor type */
	   if (-1 == SLclass_add_typecast (new_type, ancestor_type,
							opaque_typecast, 1))
		return SLANG_UNDEFINED_TYPE;

	   type = type->parent;
	}
   }

   if ( (type = (Slirp_Type*) SLmalloc(sizeof(Slirp_Type))) == NULL)
	return SLANG_UNDEFINED_TYPE;

   type->slclass = new_class;
   type->parent = parent;
   type->finalizer = fzer;
   type->initializer = izer;
   Slirp_Opaque_Types[ Slirp_Num_Opaque_Types++ ] = type;

   return new_type;
}

static int allocate_reserved_opaque_types(void)
{
   Reserved_Opaque_Type *pt;
   long abi_version = -1;

   if ( SLang_is_defined((char*)"_slirp_initialized")) {

	if ( SLang_is_defined((char*)"_slirp_abi_version") == 0  ||
		SLang_load_string((char*)"_slirp_abi_version;") == -1 ||
		SLang_pop_long(&abi_version) == -1 ||
		abi_version != SLIRP_ABI_VERSION)
	{
	   SLang_verror(SLEU,
		(char*) "SLIRP abi mismatch: want version %ld, have %ld\n",
		(long)SLIRP_ABI_VERSION, abi_version);
	   return -1;
	}

	return 0;
   }

   (void)SLang_load_string((char*)"public variable _slirp_initialized=1;");
   (void)SLang_push_int(SLIRP_ABI_VERSION);
   (void)SLang_load_string((char*)"public variable _slirp_abi_version=();");

   Slirp_Num_Opaque_Types = 0;

   if ( NULL == (Slirp_Opaque_Types = (Slirp_Type**) SLmalloc
			( sizeof(Slirp_Type*) * (NUM_RESERVED_OPAQUES + 1))))
      return -1;

   Slirp_Opaque_Types[0] = NULL;

   pt = Reserved_Opaque_Types;
   while (pt->name != NULL) {
	if ((*pt->type = allocate_opaque(pt->name, NULL, NULL, 0)) ==
	      						SLANG_UNDEFINED_TYPE)
		return -1;
	   pt++;
	}

   /* Loop again, to enable casts to/from generic pointer type */
   pt = Reserved_Opaque_Types + 1;
   while (pt->name != NULL) {

	if (-1 == SLclass_add_typecast (*pt->type, void_ptr_Type,
							opaque_typecast, 1))
	   return SLANG_UNDEFINED_TYPE;

	if (-1 == SLclass_add_typecast (void_ptr_Type, *pt->type,
							opaque_typecast, 1))
	   return SLANG_UNDEFINED_TYPE;

	pt++;
   }

   First_Opaque_Type = *Reserved_Opaque_Types[0].type;
   Last_Reserved_Opaque_Type = First_Opaque_Type + NUM_RESERVED_OPAQUES - 1;

   return 0;
} /* }}} */
static int pop_nullable(SLtype type, void **obj, void **meta)
{
   if (SLANG_NULL_TYPE == SLang_peek_at_stack ()) { 
	*obj = NULL;
	if (type && meta != NULL)
	   *meta = NULL;
	return SLang_pop_null ();
   }

   switch(type) {

	case 0:
	   return SLang_pop_cstruct(*obj, (SLang_CStruct_Field_Type *)meta);

	case SLANG_ANY_TYPE:
	   return SLang_pop_anytype((SLang_Any_Type**)obj);

	case SLANG_REF_TYPE:
	   return SLang_pop_ref((SLang_Ref_Type**)obj);

	case SLANG_STRING_TYPE:
	   return SLang_pop_slstring ((char**)obj);

	default:
	   /* This will fail, as intended, for non-opaque types */
	   return SLang_pop_opaque(type, meta, (Slirp_Opaque**)obj);
   }
}
#define pop_string_or_null(s)  pop_nullable(SLANG_STRING_TYPE,(void**)s,NULL)
#define pop_ref_or_null(r)  pop_nullable(SLANG_REF_TYPE,(void**)&r->ref,r->data)
#define pop_anytype_or_null(a) pop_nullable(SLANG_ANY_TYPE,(void**)a,NULL)

static unsigned char map_scalars_to_refs = 0;

typedef struct _Slirp_Ref {			/* Ref handling code {{{ */
#define	REF_FLAG_FREE_DATA			0x01
#define	REF_FLAG_IS_OPAQUE			0x02
#define	REF_FLAG_COLUMN_MAJOR			0x04
#define	REF_FLAG_ARRAY_EXPECTED			0x08
   unsigned int		flags;		/* Slirp_Ref encapsulates S-Lang    */
   SLtype		sltype;		/* array, ref, and MMT types, the   */
   size_t		sizeof_type;	/* latter two of which are seen as  */
   void			**data;		/* pointing to a single object      */
   unsigned int		vstride;	/* how to find "next" vectored elem */
   SLang_Array_Type	*array;
   SLang_Ref_Type	*ref;
   SLang_MMT_Type	*mmt;		/* mmt supports passing around C    */
} Slirp_Ref;				/* ptr arrays of indeterminate size */

static Slirp_Ref* ref_new(SLtype t,size_t typesize, void *d,unsigned int flags)
{
   Slirp_Ref *ref;
   if ((ref = (Slirp_Ref*)SLcalloc( sizeof(Slirp_Ref), 1)) != NULL) {
	ref->sltype = t;
	ref->sizeof_type = typesize;
	ref->flags = flags;
	ref->data = (void**)d;
	*ref->data = NULL;
   }
   return ref;
}

static int ref_finalize(Slirp_Ref *r) /* {{{ */
{
   int status = 0;
   if (r == NULL) return 0;

   if (r->ref) {

        void *ref_value = NULL; SLtype ref_type = 0; double dc[2];

	if (r->flags & REF_FLAG_IS_OPAQUE) { 	/* wrap aggregates/opaques  */
#ifdef NUM_RESERVED_OPAQUES			/* in mmt before ref assign */
	   void *opaqval = *r->data;		/* the mmt w/be freed when  */
	   if (opaqval == NULL) {		/* the S-Lang object goes   */
		ref_value = NULL;		/* out of scope		    */
		ref_type = SLANG_NULL_TYPE;
	   }
	   else {
		SLang_MMT_Type *mmt = create_opaque_mmt(r->sltype, opaqval, 0);
		ref_value = &mmt;
		ref_type = r->sltype;
	   }
#endif
	}
	else {
	   ref_type = r->sltype;
	   if (ref_type == SLANG_COMPLEX_TYPE && r->sizeof_type < sizeof(dc)) {
		float *fc = (float*) r->data;
		dc[0] = fc[0];
		dc[1] = fc[1];
		ref_value = dc;
	   }
	   else
		ref_value = r->data;
	}

	status = SLang_assign_to_ref (r->ref, ref_type, ref_value);
	SLang_free_ref(r->ref);
   }
   else if (r->array) {
#ifdef HAVE_FORTRAN_CODE
	if ((r->flags & REF_FLAG_COLUMN_MAJOR) && TRANSPOSE(1,r->array) == -1)
		return -1;
#endif
	SLang_free_array(r->array);
   }
   else if (r->mmt)
	SLang_free_mmt(r->mmt);

   if (r->flags & REF_FLAG_FREE_DATA)
	SLfree( (char*) r->data );

   SLfree((char*)r);
   return status;
} /* }}} */

static void finalize_refs(unsigned int nargs, ...) /* {{{ */
{
   va_list ap;
   va_start(ap, nargs);
   while (nargs--) ref_finalize(va_arg(ap, Slirp_Ref *));
   va_end(ap);
} /* }}} */

static unsigned int ref_get_size(Slirp_Ref *r, int which_dimension) /* {{{ */
{
   if (0) (void) &ref_get_size;	/* avoid compiler warnings if unused */

   if (r->array) {
	if (which_dimension == 0)
	   return (unsigned int)r->array->num_elements;
	else if (which_dimension < 0)
	   return r->array->num_dims;
	else {
	   which_dimension--;
	   if ((unsigned int)which_dimension < r->array->num_dims)
		return (unsigned int)r->array->dims[which_dimension];
	   else
		return 0;
	}
   }

   return 1;
} /* }}} */

extern LINKAGE int _SLang_get_class_type (SLtype t);	/* quasi-public  */

#ifdef NUM_RESERVED_OPAQUES
static SLtype sltype_to_opaque_ptr_type(SLtype sltype) /*{{{*/
{
   Reserved_Opaque_Type *pt;

   if (sltype > Last_Reserved_Opaque_Type) return opaque_ptr_Type;
   if (sltype == void_ptr_Type) return void_ptr_Type;

   pt = Reserved_Opaque_Types;		/* sequential search, but s/b < O(n) */
   while (pt->name) {			/* since list is ordered by expected */
	if (pt->masked_type == sltype)	/* frequency of use for each SLtype  */
	   return *pt->type;		
	pt++;
   }
   return 0;
} /*}}}*/
#endif

static int try_pop_mmt(SLtype type, SLang_MMT_Type **mmt) /*{{{*/
{
   static SLang_Name_Type *cl_type_func;	/* SLang_pop_mmt doesn't */
   int classtype;				/* validate that type is */
						/* an MMT, so we do here */
   *mmt = NULL;					/* FIXME: remove v2.0.7  */

   if (cl_type_func == NULL)
	cl_type_func = SLang_get_function( (char*) "__class_type");

   if (cl_type_func == NULL)
      return 0;

   if (-1 == SLang_push_datatype(type) ||		/* do the hard way, */
	-1 == SLexecute_function(cl_type_func) ||	/* as C api lacks   */
	-1 == SLang_pop_int(&classtype))		/* get_class_type() */
	return -1;

   if (classtype == SLANG_CLASS_TYPE_MMT) {
	*mmt = SLang_pop_mmt(type);
	return 1;
   }
   return 0;
} /*}}}*/

#define POP_FLAG_NULLABLE	0x1
#define POP_FLAG_VECTORIZE	0x2
static int pop_array_or_ref(Slirp_Ref *r, int flags, int defaultable) /*{{{*/
{
   SLtype type;
   unsigned int i, objtype;
#ifdef NUM_RESERVED_OPAQUES
   unsigned int is_opaque;
#endif

   if (r == NULL) {
	SLang_verror(SLEI, (char*)"Attempted NULL reference (out of memory?)");
	return -1;
   }

   if (defaultable && SLang_Num_Function_Args < defaultable) {
	r->ref   = NULL;	/* observe that only NULL can be */
	*r->data = NULL;	/* assigned as the default value */
	return 0;
   }
   
   objtype = SLang_peek_at_stack();

   if ((flags & POP_FLAG_NULLABLE) && objtype == SLANG_NULL_TYPE) {
	r->ref   = NULL;		/* nullable flag: a pointer arg for */
	*r->data = NULL;		/* which NULL is a legitimate value */
	return SLang_pop_null ();
   }

   type = r->sltype;

#ifdef NUM_RESERVED_OPAQUES
   is_opaque =(type >= First_Opaque_Type && sltype_to_slirp_type(type) != NULL);
   if (is_opaque) r->flags |= REF_FLAG_IS_OPAQUE;
#endif

   switch(objtype) {

	case SLANG_ARRAY_TYPE:

	   if (SLang_pop_array_of_type(&r->array, type) == -1)
		return -1;

#ifdef HAVE_FORTRAN_CODE
	   if (r->flags & REF_FLAG_COLUMN_MAJOR) {
		if (flags & POP_FLAG_VECTORIZE)		/* vectorizable arrs*/
		   r->flags ^= REF_FLAG_COLUMN_MAJOR;   /* r not transposed */
		else if (TRANSPOSE(0,r->array) == -1)
		   return -1;
	   }
#endif

	   i = r->array->num_elements;
#ifdef NUM_RESERVED_OPAQUES
	   if (is_opaque) {
	      	Slirp_Opaque *ot;
		SLang_MMT_Type** mmts = (SLang_MMT_Type**)r->array->data;
		void **arr = (void**)SLmalloc(i * sizeof(void*) );
		if (arr == NULL) return -1;

		while (i--) {
		   ot = (Slirp_Opaque*) SLang_object_from_mmt (mmts[i]);
		   if (ot == NULL) {
			SLfree((char*)arr);
			return -1;
		   }
		   arr[i] = ot->instance;
		}

		*r->data = (void*)arr; r->data = (void**)arr;
		r->flags |= REF_FLAG_FREE_DATA;
	   }
	   else
#endif
	   if (type == SLANG_COMPLEX_TYPE &&
		 		r->sizeof_type < r->array->sizeof_type) {
		double *dc = (double*) r->array->data;
		float  *fc = (float *) SLmalloc(i * r->sizeof_type);
		if (fc == NULL) return -1;
		*r->data = fc; r->data = (void**)fc;
		while (i--) { *fc++ = (float) *dc++; *fc++ = (float) *dc++; }
		r->flags |= REF_FLAG_FREE_DATA;
	   }
	   else {
		*r->data = r->array->data;
		r->data = (void**)*r->data;
	   }

	   break;

	case SLANG_REF_TYPE:

	   /* Refs can only send values one-way (C to S-Lang, not reverse) */
	   if (SLang_pop_ref(&r->ref) == -1)
		return -1;

	   /* Ref is assumed to point to a scalar instance of the  */
	   /* refd type, so declare enough space to hold one such. */
	   *r->data = (void*)SLmalloc(r->sizeof_type);
	   if (*r->data == NULL) return -1;
	   memset(*r->data, 0, r->sizeof_type);
	   r->flags |= REF_FLAG_FREE_DATA;
	   r->data = (void**)*r->data;
	   break;

	/* Allow scalars to used as if they were 1-element arrays */
	case SLANG_CHAR_TYPE: case SLANG_UCHAR_TYPE:
	case SLANG_SHORT_TYPE: case SLANG_USHORT_TYPE:
	case SLANG_INT_TYPE: case SLANG_UINT_TYPE:
	case SLANG_LONG_TYPE: case SLANG_ULONG_TYPE:
	case SLANG_FLOAT_TYPE: case SLANG_DOUBLE_TYPE:
	case SLANG_COMPLEX_TYPE: case SLANG_STRING_TYPE:

	   /* Accomodate FORTRAN-style pass by reference semantics */
	   if (map_scalars_to_refs &&
		 	SLang_pop_array_of_type(&r->array,type) == 0) {

		*r->data = (void*)SLmalloc(r->sizeof_type);
		if (*r->data == NULL) return -1;

		if (r->sizeof_type == r->array->sizeof_type)
		   memcpy(*r->data, r->array->data, r->sizeof_type);
		else if (type == SLANG_COMPLEX_TYPE) {
		   double *dc = (double*) r->array->data;
		   float  *fc = (float*) *r->data;
		   fc[0] = (float)dc[0];
		   fc[1] = (float)dc[1];
		}
		else  {
		   SLang_verror(SL_TYPE_MISMATCH, (char*)
			"mismatched type sizes, when popping scalar as ref");
		   SLang_free_array(r->array);
		   return -1;
		}

		r->data = (void**)*r->data;
		r->flags |= REF_FLAG_FREE_DATA;
		/* Nullify to distinguish between vectored/non-vectored args */
		SLang_free_array(r->array); r->array = NULL;
		break;
	   }				/* intentional fallthrough */

	default:

#ifdef NUM_RESERVED_OPAQUES
	   if (objtype >= First_Opaque_Type && 
		 		sltype_to_slirp_type(objtype) != NULL) {

		if (!(flags & POP_FLAG_VECTORIZE))
		   type = sltype_to_opaque_ptr_type(type);

		if (type) {

		   Slirp_Opaque *otp;
		   if (SLang_pop_opaque(type, NULL, &otp) == -1)
			return -1;
		
		   if (flags & POP_FLAG_VECTORIZE) {
			void **arr = (void**) SLmalloc(sizeof(void*));
			if (arr == NULL) return -1;
			arr[0] = otp->instance;
			*r->data = arr;
			r->flags |= REF_FLAG_FREE_DATA;
		   }
		   else
			*r->data = otp->instance;

		   r->data = (void**)*r->data;
		   r->mmt = otp->mmt;
		   return 0;
		}
	   }
	   else
#endif
	   if ( try_pop_mmt(objtype, &r->mmt) == 1 &&
		(*r->data = SLang_object_from_mmt (r->mmt)) != NULL) {
		   r->data = (void**)*r->data;	 /* not flagged for freeing */
		   return 0;
	   }

	   SLang_verror(SL_TYPE_MISMATCH, (char*)
			"context requires array, ref, or opaque pointer");
	   return -1;
   }
   return 0;
} /*}}}*/
/* }}} */
static void push_null_term_str_array(char **arr, char *funcname, int free_arr)
{
   int i;
   char **copy = arr;
   SLang_Array_Type *sarr;

   while (*copy++) ;
   i = copy - arr - 1;

   sarr = SLang_create_array (SLANG_STRING_TYPE, 0, NULL, &i, 1);
   if (sarr != NULL) {
	   
	char **slstr = (char**)sarr->data;
	while (i--)
	   if ( NULL == (slstr[i] = SLang_create_slstring (arr[i]))) {
		SLang_free_array(sarr);
		sarr = NULL;
		break;
	   }

   }
   if (sarr == NULL) {
	if (funcname == NULL) funcname = "unknown function";
	SLang_verror(SLEI, "creating string array in %s", funcname);
   }
   else
	SLang_push_array(sarr, 1);

   if (free_arr) {
	i = copy - arr;
	while (i--)
	   SLfree(arr[i]);
	SLfree((char*)arr);
   }
}

/* Wrapper functions */ /* {{{ */
static void sl_gtk_databox_get_enable_zoom (void)
{
   gboolean retval;
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;

   if (SLang_Num_Function_Args != 1 ||
	SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
	{Slirp_usage(0,0,0); return;}

   retval = gtk_databox_get_enable_zoom(arg1);
   SLang_push_int(retval);
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_grid_set_vlines (void)
{
   GtkDataboxGrid* arg1;
   Slirp_Opaque* arg1_o = NULL;
   gint arg2;

   if (SLang_Num_Function_Args != 2 ||
	SLang_pop_int((int*)&arg2) == -1 ||
	SLang_pop_opaque(GtkOpaque_Type, (void**)&arg1, &arg1_o) == -1 )
	{Slirp_usage(1,1,0); return;}

   gtk_databox_grid_set_vlines(arg1,  arg2);
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_cross_simple_new (void)
{
   GtkDataboxGraph* retval;
   GdkColor* arg1 = (GdkColor*) alloca(sizeof(GdkColor));
   guint arg2;

   if (SLang_Num_Function_Args != 2 ||
	SLang_pop_uint((unsigned int*)&arg2) == -1 ||
	SLang_pop_cstruct ( (VOID_STAR)arg1, GdkColor_Layout) == -1 )
	{Slirp_usage(2,2,0); return;}

   retval = gtk_databox_cross_simple_new(arg1,  arg2);
   SLang_push_opaque(GtkOpaque_Type, (void*)retval, 0);
   SLang_free_cstruct((VOID_STAR)arg1, GdkColor_Layout);
}

static void sl_gtk_databox_get_ruler_x (void)
{
   GtkDataboxRuler* retval;
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;

   if (SLang_Num_Function_Args != 1 ||
	SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
	{Slirp_usage(3,3,0); return;}

   retval = gtk_databox_get_ruler_x(arg1);
   SLang_push_opaque(GtkWidget_Type, (void*)retval, 0);
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_get_adjustment_y (void)
{
   GtkAdjustment* retval;
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;

   if (SLang_Num_Function_Args != 1 ||
	SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
	{Slirp_usage(4,4,0); return;}

   retval = gtk_databox_get_adjustment_y(arg1);
   SLang_push_opaque(GtkAdjustment_Type, (void*)retval, 0);
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_create_box_with_scrollbars_and_rulers (void)
{   
   gboolean arg1, arg2, arg3, arg4;
   GtkWidget* databox;
   GtkWidget *table;
   
   if (SLang_Num_Function_Args != 4 ||
       SLang_pop_int((int*)&arg4) == -1 ||
       SLang_pop_int((int*)&arg3) == -1 ||
       SLang_pop_int((int*)&arg2) == -1 ||
       SLang_pop_int((int*)&arg1) == -1)
     {Slirp_usage(5,5,0); return;}

   gtk_databox_create_box_with_scrollbars_and_rulers (&databox, &table, arg1, arg2, arg3, arg4);
   SLang_push_opaque (GtkWidget_Type, (void*)databox, 0);
   SLang_push_opaque (GtkWidget_Type, (void*)table, 0);
}

static void sl_gtk_databox_set_adjustment_x (void)
{
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;
   GtkAdjustment* arg2;
   Slirp_Opaque* arg2_o = NULL;

   if (SLang_Num_Function_Args != 2 ||
	SLang_pop_opaque(GtkAdjustment_Type, (void**)&arg2, &arg2_o) == -1 ||
	SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
	{Slirp_usage(6,6,0); return;}

   gtk_databox_set_adjustment_x(arg1, arg2);
   SLang_free_opaque(arg1_o);
   SLang_free_opaque(arg2_o);
}

static void sl_gtk_databox_get_enable_selection (void)
{
   gboolean retval;
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;

   if (SLang_Num_Function_Args != 1 ||
	SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
	{Slirp_usage(7,7,0); return;}

   retval = gtk_databox_get_enable_selection(arg1);
   SLang_push_int(retval);
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_get_adjustment_x (void)
{
   GtkAdjustment* retval;
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;

   if (SLang_Num_Function_Args != 1 ||
	SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
	{Slirp_usage(8,8,0); return;}

   retval = gtk_databox_get_adjustment_x(arg1);
   SLang_push_opaque(GtkAdjustment_Type, (void*)retval, 0);
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_set_enable_zoom (void)
{
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;
   gboolean arg2;

   if (SLang_Num_Function_Args != 2 ||
	SLang_pop_int((int*)&arg2) == -1 ||
	SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
	{Slirp_usage(9,9,0); return;}

   gtk_databox_set_enable_zoom(arg1,  arg2);
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_set_ruler_y (void)
{
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;
   GtkDataboxRuler* arg2;
   Slirp_Opaque* arg2_o = NULL;

   if (SLang_Num_Function_Args != 2 ||
	SLang_pop_opaque(GtkWidget_Type, (void**)&arg2, &arg2_o) == -1 ||
	SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
	{Slirp_usage(10,10,0); return;}

   gtk_databox_set_ruler_y(arg1, arg2);
   SLang_free_opaque(arg1_o);
   SLang_free_opaque(arg2_o);
}

static void sl_gtk_databox_set_adjustment_y (void)
{
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;
   GtkAdjustment* arg2;
   Slirp_Opaque* arg2_o = NULL;

   if (SLang_Num_Function_Args != 2 ||
	SLang_pop_opaque(GtkAdjustment_Type, (void**)&arg2, &arg2_o) == -1 ||
	SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
	{Slirp_usage(11,11,0); return;}

   gtk_databox_set_adjustment_y(arg1, arg2);
   SLang_free_opaque(arg1_o);
   SLang_free_opaque(arg2_o);
}

static void sl_gtk_databox_zoom_out (void)
{
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;

   if (SLang_Num_Function_Args != 1 ||
	SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
	{Slirp_usage(12,12,0); return;}

   gtk_databox_zoom_out(arg1);
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_get_ruler_y (void)
{
   GtkDataboxRuler* retval;
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;

   if (SLang_Num_Function_Args != 1 ||
	SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
	{Slirp_usage(13,13,0); return;}

   retval = gtk_databox_get_ruler_y(arg1);
   SLang_push_opaque(GtkWidget_Type, (void*)retval, 0);
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_graph_remove_all (void)
{
   gint retval;
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;

   if (SLang_Num_Function_Args != 1 ||
	SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
	{Slirp_usage(14,14,0); return;}

   retval = gtk_databox_graph_remove_all(arg1);
   SLang_push_int (retval);
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_grid_get_vlines (void)
{
   gint retval;
   GtkDataboxGrid* arg1;
   Slirp_Opaque* arg1_o = NULL;

   if (SLang_Num_Function_Args != 1 ||
	SLang_pop_opaque(GtkOpaque_Type, (void**)&arg1, &arg1_o) == -1 )
	{Slirp_usage(15,15,0); return;}

   retval = gtk_databox_grid_get_vlines(arg1);
   SLang_push_int(retval);
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_set_ruler_x (void)
{
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;
   GtkDataboxRuler* arg2;
   Slirp_Opaque* arg2_o = NULL;

   if (SLang_Num_Function_Args != 2 ||
	SLang_pop_opaque(GtkWidget_Type, (void**)&arg2, &arg2_o) == -1 ||
	SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
	{Slirp_usage(16,16,0); return;}

   gtk_databox_set_ruler_x(arg1, arg2);
   SLang_free_opaque(arg1_o);
   SLang_free_opaque(arg2_o);
}

static void sl_gtk_databox_grid_new (void)
{
   GtkDataboxGraph* retval;
   gint arg1;
   gint arg2;
   GdkColor* arg3 = (GdkColor*) alloca(sizeof(GdkColor));
   guint arg4;

   if (SLang_Num_Function_Args != 4 ||
	SLang_pop_uint((unsigned int*)&arg4) == -1 ||
	SLang_pop_cstruct ( (VOID_STAR)arg3, GdkColor_Layout) == -1 ||
	SLang_pop_int((int*)&arg2) == -1 ||
	SLang_pop_int((int*)&arg1) == -1 )
	{Slirp_usage(17,17,0); return;}

   retval = gtk_databox_grid_new( arg1,  arg2, arg3,  arg4);
   SLang_push_opaque(GtkOpaque_Type, (void*)retval, 0);
   SLang_free_cstruct((VOID_STAR)arg3, GdkColor_Layout);
}

static void sl_gtk_databox_markers_set_label (void)
{
   GtkDataboxGraph* arg1;
   Slirp_Opaque* arg1_o = NULL;
   guint arg2;
   GtkDataboxMarkersTextPosition arg3;
   gchar* arg4;
   gboolean arg5;

   if (SLang_Num_Function_Args != 5 ||
	SLang_pop_int((int*)&arg5) == -1 ||
	SLang_pop_string((char**)&arg4) == -1 ||
	SLang_pop_int((int*)&arg3) == -1 ||
	SLang_pop_uint((unsigned int*)&arg2) == -1 ||
	SLang_pop_opaque(GtkOpaque_Type, (void**)&arg1, &arg1_o) == -1 )
	{Slirp_usage(18,18,0); return;}

   gtk_databox_markers_set_label (GTK_DATABOX_MARKERS (arg1),  arg2,  arg3, arg4,  arg5);
   SLang_free_opaque(arg1_o);
   SLang_free_slstring(arg4);
}

static void sl_gtk_databox_zoom_to_selection (void)
{
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;

   if (SLang_Num_Function_Args != 1 ||
	SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
	{Slirp_usage(19,19,0); return;}

   gtk_databox_zoom_to_selection(arg1);
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_get_visible_limits (void)
{
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;
   gfloat left, right, top, bottom;
   
   if (SLang_Num_Function_Args != 1 ||
       SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
     {Slirp_usage(20,20,0); return;}
   
   gtk_databox_get_visible_limits (arg1, &left, &right, &top, &bottom);

   (void) SLang_push_float (left);
   (void) SLang_push_float (right);
   (void) SLang_push_float (top);
   (void) SLang_push_float (bottom);
   SLang_free_opaque(arg1_o);   
}

static void sl_gtk_databox_set_total_limits (void)
{
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;
   gfloat left, right, top, bottom;

   if (SLang_Num_Function_Args != 5 ||
       SLang_pop_float((float*)&bottom) == -1 ||
       SLang_pop_float((float*)&top) == -1 ||
       SLang_pop_float((float*)&right) == -1 ||
       SLang_pop_float((float*)&left) == -1 ||
       SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
     {Slirp_usage(21,21,0); return;}

   gtk_databox_set_total_limits (arg1, left, right, top, bottom);
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_set_enable_selection (void)
{
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;
   gboolean arg2;

   if (SLang_Num_Function_Args != 2 ||
	SLang_pop_int((int*)&arg2) == -1 ||
	SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
	{Slirp_usage(22,22,0); return;}

   gtk_databox_set_enable_selection(arg1,  arg2);
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_lines_new (void)
{
   GtkDataboxGraph* retval;
   GdkColor* arg1 = (GdkColor*) alloca(sizeof(GdkColor));
   guint arg2;
   SLang_Array_Type *x, *y;
   
   if (SLang_Num_Function_Args != 4 ||
       SLang_pop_uint((unsigned int*)&arg2) == -1 ||
       SLang_pop_cstruct ( (VOID_STAR)arg1, GdkColor_Layout) == -1 ||
       SLang_pop_array_of_type (&y, SLANG_FLOAT_TYPE) == -1 ||
       SLang_pop_array_of_type (&x, SLANG_FLOAT_TYPE) == -1)
     {Slirp_usage(23,23,0); return;}

   retval = gtk_databox_lines_new (x->dims [0], x->data, y->data, arg1, arg2);
   SLang_push_opaque(GtkOpaque_Type, (void*)retval, 0);
   SLang_free_cstruct((VOID_STAR)arg1, GdkColor_Layout);
   /* SLang_free_array (x); */
   /* SLang_free_array (y); */
}

static void sl_gtk_databox_set_visible_limits (void)
{
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;
   gfloat left, right, top, bottom;

   if (SLang_Num_Function_Args != 5 ||
       SLang_pop_float((float*)&bottom) == -1 ||
       SLang_pop_float((float*)&top) == -1 ||
       SLang_pop_float((float*)&right) == -1 ||
       SLang_pop_float((float*)&left) == -1 ||
       SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
     {Slirp_usage(24,24,0); return;}

   gtk_databox_set_visible_limits (arg1, left, right, top, bottom);
   SLang_free_opaque(arg1_o);

}

static void sl_gtk_databox_graph_add (void)
{
   gint retval;
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;
   GtkDataboxGraph* arg2;
   Slirp_Opaque* arg2_o = NULL;

   if (SLang_Num_Function_Args != 2 ||
	SLang_pop_opaque(GtkOpaque_Type, (void**)&arg2, &arg2_o) == -1 ||
	SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
	{Slirp_usage(25,25,0); return;}

   retval = gtk_databox_graph_add(arg1, arg2);
   SLang_push_int(retval);
   SLang_free_opaque(arg1_o);
   SLang_free_opaque(arg2_o);
}

static void sl_gtk_databox_grid_set_hlines (void)
{
   GtkDataboxGrid* arg1;
   Slirp_Opaque* arg1_o = NULL;
   gint arg2;

   if (SLang_Num_Function_Args != 2 ||
	SLang_pop_int((int*)&arg2) == -1 ||
	SLang_pop_opaque(GtkOpaque_Type, (void**)&arg1, &arg1_o) == -1 )
	{Slirp_usage(26,26,0); return;}

   gtk_databox_grid_set_hlines(arg1,  arg2);
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_markers_set_position (void)
{
   GtkDataboxGraph* arg1;   
   Slirp_Opaque* arg1_o = NULL;
   guint arg2;
   GtkDataboxMarkersPosition arg3;
   
   if (SLang_Num_Function_Args != 3 ||
	SLang_pop_int((int*)&arg3) == -1 ||
	SLang_pop_uint((unsigned int*)&arg2) == -1 ||
	SLang_pop_opaque(GtkOpaque_Type, (void**)&arg1, &arg1_o) == -1 )
	{Slirp_usage(27,27,0); return;}

   gtk_databox_markers_set_position (GTK_DATABOX_MARKERS (arg1),  arg2,  arg3);
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_calculate_extrema (void)
{
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;
   gfloat min_x, max_x, min_y, max_y;

   if (SLang_Num_Function_Args != 1 ||
       SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
     {Slirp_usage(28,28,0); return;}
   
   gtk_databox_calculate_extrema (arg1, &min_x, &max_x, &min_y, &max_y);

   (void) SLang_push_float (min_x);
   (void) SLang_push_float (max_x);
   (void) SLang_push_float (min_y);
   (void) SLang_push_float (min_y);
   SLang_free_opaque(arg1_o);   
}

static void sl_gtk_databox_graph_remove (void)
{
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;
   GtkDataboxGraph* arg2;
   Slirp_Opaque* arg2_o = NULL;

   if (SLang_Num_Function_Args != 2 ||
	SLang_pop_opaque(GtkOpaque_Type, (void**)&arg2, &arg2_o) == -1 ||
	SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
	{Slirp_usage(29,29,0); return;}

   gtk_databox_graph_remove(arg1, arg2);
   SLang_free_opaque(arg1_o);
   SLang_free_opaque(arg2_o);
}

static void sl_gtk_databox_markers_new (void)
{
   GtkDataboxGraph* retval;
   GdkColor* arg1 = (GdkColor*) alloca(sizeof(GdkColor));
   guint arg2;
   GtkDataboxMarkersType arg3;
   SLang_Array_Type *x, *y;
   
   if (SLang_Num_Function_Args != 5 ||
       SLang_pop_int((int*)&arg3) == -1 ||
       SLang_pop_uint((unsigned int*)&arg2) == -1 ||
       SLang_pop_cstruct ( (VOID_STAR)arg1, GdkColor_Layout) == -1 ||
       SLang_pop_array_of_type (&y, SLANG_FLOAT_TYPE) == -1 ||
       SLang_pop_array_of_type (&x, SLANG_FLOAT_TYPE) == -1)
     {Slirp_usage(30,30,0); return;}

   retval = gtk_databox_markers_new (x->dims [0], x->data, y->data, arg1, arg2, arg3);
   SLang_push_opaque(GtkOpaque_Type, (void*)retval, 0);
   SLang_free_cstruct((VOID_STAR)arg1, GdkColor_Layout);
}

static void sl_gtk_databox_get_total_limits (void)
{
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;
   gfloat left, right, top, bottom;
   
   if (SLang_Num_Function_Args != 1 ||
       SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
     {Slirp_usage(31,31,0); return;}
   
   gtk_databox_get_total_limits (arg1, &left, &right, &top, &bottom);

   (void) SLang_push_float (left);
   (void) SLang_push_float (right);
   (void) SLang_push_float (top);
   (void) SLang_push_float (bottom);
   SLang_free_opaque(arg1_o);   
}

static void sl_gtk_databox_new (void)
{
   GtkWidget* retval;
   if (SLang_Num_Function_Args != 0)
	{Slirp_usage(34,34,0); return;}

   retval = gtk_databox_new();
   SLang_push_opaque(GtkWidget_Type, (void*)retval, 0);
}

static void sl_gtk_databox_grid_get_hlines (void)
{
   gint retval;
   GtkDataboxGrid* arg1;
   Slirp_Opaque* arg1_o = NULL;

   if (SLang_Num_Function_Args != 1 ||
	SLang_pop_opaque(GtkOpaque_Type, (void**)&arg1, &arg1_o) == -1 )
	{Slirp_usage(32,32,0); return;}

   retval = gtk_databox_grid_get_hlines(arg1);
   SLang_push_int(retval);
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_zoom_home (void)
{
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;

   if (SLang_Num_Function_Args != 1 ||
	SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
	{Slirp_usage(33,33,0); return;}

   gtk_databox_zoom_home(arg1);
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_bars_new (void)
{
   GtkDataboxGraph* retval;
   GdkColor* arg1 = (GdkColor*) alloca(sizeof(GdkColor));
   guint arg2;
   SLang_Array_Type *x, *y;
   
   if (SLang_Num_Function_Args != 4 ||
       SLang_pop_uint((unsigned int*)&arg2) == -1 ||
       SLang_pop_cstruct ( (VOID_STAR)arg1, GdkColor_Layout) == -1 ||
       SLang_pop_array_of_type (&y, SLANG_FLOAT_TYPE) == -1 ||
       SLang_pop_array_of_type (&x, SLANG_FLOAT_TYPE) == -1)
     {Slirp_usage(35,35,0); return;}

   retval = gtk_databox_bars_new (x->dims [0], x->data, y->data, arg1, arg2);
   SLang_push_opaque(GtkOpaque_Type, (void*)retval, 0);
   SLang_free_cstruct((VOID_STAR)arg1, GdkColor_Layout);
}

static void sl_gtk_databox_points_new (void)
{
   GtkDataboxGraph* retval;
   GdkColor* arg1 = (GdkColor*) alloca(sizeof(GdkColor));
   guint arg2;
   SLang_Array_Type *x, *y;
   
   if (SLang_Num_Function_Args != 4 ||
       SLang_pop_uint((unsigned int*)&arg2) == -1 ||
       SLang_pop_cstruct ( (VOID_STAR)arg1, GdkColor_Layout) == -1 ||
       SLang_pop_array_of_type (&y, SLANG_FLOAT_TYPE) == -1 ||
       SLang_pop_array_of_type (&x, SLANG_FLOAT_TYPE) == -1)
     {Slirp_usage(36,36,0); return;}

   retval = gtk_databox_points_new (x->dims [0], x->data, y->data, arg1, arg2);
   SLang_push_opaque(GtkOpaque_Type, (void*)retval, 0);
   SLang_free_cstruct((VOID_STAR)arg1, GdkColor_Layout);
}

static void sl_gtk_databox_auto_rescale (void)
{
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;
   gfloat arg2;

   if (SLang_Num_Function_Args != 2 ||
	SLang_pop_float((float*)&arg2) == -1 ||
	SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
	{Slirp_usage(37,37,0); return;}

   gtk_databox_auto_rescale(arg1,  arg2);
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_graph_set_hide (void)
{
   GtkDataboxGraph* arg1;
   Slirp_Opaque* arg1_o = NULL;
   gboolean arg2;
   
   if (SLang_Num_Function_Args != 2 ||
       SLang_pop_int((int*)&arg2) == -1 ||
       SLang_pop_opaque(GtkOpaque_Type, (void**)&arg1, &arg1_o) == -1)
     {Slirp_usage(38,38,0); return;}

   gtk_databox_graph_set_hide (arg1, arg2);
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_graph_get_hide (void)
{   
   gboolean retval;
   GtkDataboxGraph* arg1;
   Slirp_Opaque* arg1_o = NULL;
   
   if (SLang_Num_Function_Args != 1 ||
       SLang_pop_opaque(GtkOpaque_Type, (void**)&arg1, &arg1_o) == -1 )
     {Slirp_usage(39,39,0); return;}
   
   retval = gtk_databox_graph_get_hide(arg1);
   SLang_push_int(retval);
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_graph_get_size (void)
{   
   gint retval;
   GtkDataboxGraph* arg1;
   Slirp_Opaque* arg1_o = NULL;
   
   if (SLang_Num_Function_Args != 1 ||
       SLang_pop_opaque(GtkOpaque_Type, (void**)&arg1, &arg1_o) == -1 )
     {Slirp_usage(40,40,0); return;}

      retval = gtk_databox_graph_get_size(arg1);
      SLang_push_int(retval);
      SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_graph_get_color (void)
{   
   GdkColor* retval;
   GtkDataboxGraph* arg1;
   Slirp_Opaque* arg1_o = NULL;
   
   if (SLang_Num_Function_Args != 1 ||
       SLang_pop_opaque(GtkOpaque_Type, (void**)&arg1, &arg1_o) == -1 )
     {Slirp_usage(41,41,0); return;}

   retval = gtk_databox_graph_get_color(arg1);
   SLang_push_cstruct((VOID_STAR)retval,GdkColor_Layout);
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_graph_calculate_extrema (void)
{
   GtkDataboxGraph* arg1;
   Slirp_Opaque* arg1_o = NULL;
   gfloat min_x, max_x, min_y, max_y;
   
   if (SLang_Num_Function_Args != 1 ||
       SLang_pop_opaque(GtkOpaque_Type, (void**)&arg1, &arg1_o) == -1 )
     {Slirp_usage(42,42,0); return;}
   
   gtk_databox_graph_calculate_extrema (arg1, &min_x, &max_x, &min_y, &max_y);

   (void) SLang_push_float (min_x);
   (void) SLang_push_float (max_x);
   (void) SLang_push_float (min_y);
   (void) SLang_push_float (min_y);   
   SLang_free_opaque(arg1_o);   
}

static void sl_gtk_databox_graph_set_color (void)
{   
   GtkDataboxGraph* arg1;
   Slirp_Opaque* arg1_o = NULL;
   GdkColor* arg2 = (GdkColor*) alloca(sizeof(GdkColor));
   
   if (SLang_Num_Function_Args != 2 ||
       SLang_pop_cstruct ( (VOID_STAR)arg2, GdkColor_Layout) == -1 ||
       SLang_pop_opaque(GtkOpaque_Type, (void**)&arg1, &arg1_o) == -1 )
     {Slirp_usage(43,43,0); return;}

   gtk_databox_graph_set_color(arg1, arg2);
   SLang_free_opaque(arg1_o);
   SLang_free_cstruct((VOID_STAR)arg2, GdkColor_Layout);
}

static void sl_gtk_databox_graph_set_size (void)
{   
   GtkDataboxGraph* arg1;
   Slirp_Opaque* arg1_o = NULL;
   gint arg2;
   
   if (SLang_Num_Function_Args != 2 ||
       SLang_pop_int((int*)&arg2) == -1 ||
       SLang_pop_opaque(GtkOpaque_Type, (void**)&arg1, &arg1_o) == -1 )
     {Slirp_usage(44,44,0); return;}

   gtk_databox_graph_set_size(arg1,  arg2);
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_set_scale_type_x (void)
{
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;
   GtkDataboxScaleType arg2;

   if (SLang_Num_Function_Args != 2 ||
       SLang_pop_int((int*)&arg2) == -1 ||
       SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
     {Slirp_usage(45,45,0); return;}

   gtk_databox_set_scale_type_x (arg1, arg2);
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_set_scale_type_y (void)
{
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;
   GtkDataboxScaleType arg2;

   if (SLang_Num_Function_Args != 2 ||
       SLang_pop_int((int*)&arg2) == -1 ||
       SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
     {Slirp_usage(46,46,0); return;}

   gtk_databox_set_scale_type_x (arg1, arg2);
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_get_scale_type_x (void)
{
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;

   if (SLang_Num_Function_Args != 1 ||
       SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
     {Slirp_usage(47,47,0); return;}

   SLang_push_int (gtk_databox_get_scale_type_x (arg1));
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_get_scale_type_y (void)
{
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;

   if (SLang_Num_Function_Args != 1 ||
       SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
     {Slirp_usage(48,48,0); return;}

   SLang_push_int (gtk_databox_get_scale_type_y (arg1));
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_ruler_new (void)
{
   GtkOrientation arg1;
   GtkWidget* retval;
   
   if (SLang_Num_Function_Args != 1 ||
       SLang_pop_int((int*)&arg1) == -1)
     {Slirp_usage(49,49,0); return;}

   retval = gtk_databox_ruler_new (arg1);
   SLang_push_opaque(GtkWidget_Type, (void*)retval, 0);   
}

static void sl_gtk_databox_ruler_set_range (void)
{
   GtkDataboxRuler* arg1;
   Slirp_Opaque* arg1_o = NULL;
   gfloat arg2, arg3, arg4;

   if (SLang_Num_Function_Args != 4 ||
       SLang_pop_float((float*)&arg4) == -1 ||
       SLang_pop_float((float*)&arg3) == -1 ||
       SLang_pop_float((float*)&arg2) == -1 ||
       SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
     {Slirp_usage(50,50,0); return;}

   gtk_databox_ruler_set_range (arg1, arg2, arg3, arg4);

   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_ruler_set_max_length (void)
{
   GtkDataboxRuler* arg1;
   Slirp_Opaque* arg1_o = NULL;
   guint arg2;
   
   if (SLang_Num_Function_Args != 2 ||
       SLang_pop_int((int*)&arg2) == -1 ||
       SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
     {Slirp_usage(51,51,0); return;}

   gtk_databox_ruler_set_max_length (arg1, arg2);
   
   SLang_free_opaque(arg1_o);   
}

static void sl_gtk_databox_ruler_set_scale_type (void)
{
   GtkDataboxRuler* arg1;
   Slirp_Opaque* arg1_o = NULL;
   guint arg2;
   
   if (SLang_Num_Function_Args != 2 ||
       SLang_pop_int((int*)&arg2) == -1 ||
       SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
     {Slirp_usage(52,52,0); return;}
   
   gtk_databox_ruler_set_scale_type (arg1, arg2);
   
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_ruler_get_range (void)
{
   GtkDataboxRuler* arg1;
   Slirp_Opaque* arg1_o = NULL;
   gdouble lower, upper, position;
   
   if (SLang_Num_Function_Args != 1 ||
       SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
     {Slirp_usage(53,53,0); return;}
   
   gtk_databox_ruler_get_range (arg1, &lower, &upper, &position);

   SLang_push_float (lower);
   SLang_push_float (upper);
   SLang_push_float (position);
   
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_ruler_get_max_length (void)
{
   GtkDataboxRuler* arg1;
   Slirp_Opaque* arg1_o = NULL;

   if (SLang_Num_Function_Args != 1 ||
       SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
     {Slirp_usage(54,54,0); return;}

   SLang_push_uint (gtk_databox_ruler_get_max_length (arg1));
   
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_ruler_get_scale_type (void)
{
   GtkDataboxRuler* arg1;
   Slirp_Opaque* arg1_o = NULL;

   if (SLang_Num_Function_Args != 1 ||
       SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
     {Slirp_usage(55,55,0); return;}

   SLang_push_int (gtk_databox_ruler_get_scale_type (arg1));
   
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_pixel_to_value_x (void)
{
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;
   gint pixel;
   
   if (SLang_Num_Function_Args != 2 ||
       SLang_pop_int((int*)&pixel) == -1 ||
       SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
     {Slirp_usage(56,56,0); return;}
   
   
   SLang_push_float (gtk_databox_pixel_to_value_x (arg1, pixel));
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_pixel_to_value_y (void)
{
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;
   gint pixel;
   
   if (SLang_Num_Function_Args != 2 ||
       SLang_pop_int((int*)&pixel) == -1 ||
       SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
     {Slirp_usage(57,57,0); return;}
   
   
   SLang_push_float (gtk_databox_pixel_to_value_y (arg1, pixel));
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_value_to_pixel_x (void)
{
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;
   gfloat value;
   
   if (SLang_Num_Function_Args != 2 ||
       SLang_pop_float((float*)&value) == -1 ||
       SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
     {Slirp_usage(58,58,0); return;}
      
   SLang_push_int (gtk_databox_value_to_pixel_x (arg1, value));
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_value_to_pixel_y (void)
{
   GtkDatabox* arg1;
   Slirp_Opaque* arg1_o = NULL;
   gfloat value;
   
   if (SLang_Num_Function_Args != 2 ||
       SLang_pop_float((float*)&value) == -1 ||
       SLang_pop_opaque(GtkWidget_Type, (void**)&arg1, &arg1_o) == -1 )
     {Slirp_usage(59,59,0); return;}
      
   SLang_push_int (gtk_databox_value_to_pixel_y (arg1, value));
   SLang_free_opaque(arg1_o);
}

static void sl_gtk_databox_graph_set_gc (void)
{
   GtkDataboxGraph* arg1;
   Slirp_Opaque* arg1_o = NULL;
   GdkGC *arg2;
   Slirp_Opaque* arg2_o = NULL;
   
   if (SLang_Num_Function_Args != 2 ||
       SLang_pop_opaque(GdkGC_Type, (void**)&arg2, &arg2_o) == -1 ||
       SLang_pop_opaque(GtkOpaque_Type, (void**)&arg1, &arg1_o) == -1 )
     {Slirp_usage(60,60,0); return;}   
   
   gtk_databox_graph_set_gc (arg1, arg2);
   SLang_free_opaque(arg1_o);
   SLang_free_opaque(arg2_o);
}

static void sl_gtk_databox_graph_get_gc (void)
{
   GtkDataboxGraph* arg1;
   Slirp_Opaque* arg1_o = NULL;
   GdkGC *retval;
   
   if (SLang_Num_Function_Args != 1 ||
       SLang_pop_opaque(GtkOpaque_Type, (void**)&arg1, &arg1_o) == -1 )
     {Slirp_usage(61,61,0); return;}   
   
   retval = gtk_databox_graph_get_gc (arg1);
   SLang_push_opaque(GdkGC_Type, (void*)retval, 0);
   SLang_free_opaque(arg1_o);   
}

static SLang_Intrin_Fun_Type gtkdatabox_Funcs [] =
{
   MAKE_INTRINSIC_0((char*)"gtk_databox_get_enable_zoom",sl_gtk_databox_get_enable_zoom,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_grid_set_vlines",sl_gtk_databox_grid_set_vlines,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_cross_simple_new",sl_gtk_databox_cross_simple_new,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_get_ruler_x",sl_gtk_databox_get_ruler_x,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_get_adjustment_y",sl_gtk_databox_get_adjustment_y,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_create_box_with_scrollbars_and_rulers",sl_gtk_databox_create_box_with_scrollbars_and_rulers,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_set_adjustment_x",sl_gtk_databox_set_adjustment_x,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_get_enable_selection",sl_gtk_databox_get_enable_selection,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_get_adjustment_x",sl_gtk_databox_get_adjustment_x,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_set_enable_zoom",sl_gtk_databox_set_enable_zoom,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_set_ruler_y",sl_gtk_databox_set_ruler_y,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_set_adjustment_y",sl_gtk_databox_set_adjustment_y,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_zoom_out",sl_gtk_databox_zoom_out,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_get_ruler_y",sl_gtk_databox_get_ruler_y,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_graph_remove_all",sl_gtk_databox_graph_remove_all,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_grid_get_vlines",sl_gtk_databox_grid_get_vlines,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_set_ruler_x",sl_gtk_databox_set_ruler_x,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_grid_new",sl_gtk_databox_grid_new,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_markers_set_label",sl_gtk_databox_markers_set_label,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_zoom_to_selection",sl_gtk_databox_zoom_to_selection,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_get_visible_limits",sl_gtk_databox_get_visible_limits,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_set_total_limits",sl_gtk_databox_set_total_limits,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_set_enable_selection",sl_gtk_databox_set_enable_selection,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_lines_new",sl_gtk_databox_lines_new,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_set_visible_limits",sl_gtk_databox_set_visible_limits,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_graph_add",sl_gtk_databox_graph_add,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_grid_set_hlines",sl_gtk_databox_grid_set_hlines,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_markers_set_position",sl_gtk_databox_markers_set_position,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_calculate_extrema",sl_gtk_databox_calculate_extrema,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_graph_remove",sl_gtk_databox_graph_remove,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_markers_new",sl_gtk_databox_markers_new,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_get_total_limits",sl_gtk_databox_get_total_limits,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_new",sl_gtk_databox_new,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_grid_get_hlines",sl_gtk_databox_grid_get_hlines,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_zoom_home",sl_gtk_databox_zoom_home,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_bars_new",sl_gtk_databox_bars_new,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_points_new",sl_gtk_databox_points_new,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_auto_rescale",sl_gtk_databox_auto_rescale,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_graph_set_hide",sl_gtk_databox_graph_set_hide,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_graph_get_hide",sl_gtk_databox_graph_get_hide,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_graph_get_size",sl_gtk_databox_graph_get_size,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_graph_get_color",sl_gtk_databox_graph_get_color,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_graph_calculate_extrema",sl_gtk_databox_graph_calculate_extrema,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_graph_set_color",sl_gtk_databox_graph_set_color,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_graph_set_size",sl_gtk_databox_graph_set_size,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_set_scale_type_x",sl_gtk_databox_set_scale_type_x,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_set_scale_type_y",sl_gtk_databox_set_scale_type_y,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_get_scale_type_x",sl_gtk_databox_get_scale_type_x,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_get_scale_type_y",sl_gtk_databox_get_scale_type_y,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_ruler_new",sl_gtk_databox_ruler_new,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_ruler_set_range",sl_gtk_databox_ruler_set_range,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_ruler_set_max_length",sl_gtk_databox_ruler_set_max_length,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_ruler_set_scale_type",sl_gtk_databox_ruler_set_scale_type,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_ruler_get_range",sl_gtk_databox_ruler_get_range,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_ruler_get_max_length",sl_gtk_databox_ruler_get_max_length,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_ruler_get_scale_type",sl_gtk_databox_ruler_get_scale_type,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_pixel_to_value_x",sl_gtk_databox_pixel_to_value_x,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_pixel_to_value_y",sl_gtk_databox_pixel_to_value_y,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_value_to_pixel_x",sl_gtk_databox_value_to_pixel_x,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_value_to_pixel_y",sl_gtk_databox_value_to_pixel_y,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_graph_set_gc",sl_gtk_databox_graph_set_gc,V),
   MAKE_INTRINSIC_0((char*)"gtk_databox_graph_get_gc",sl_gtk_databox_graph_get_gc,V),
   SLANG_END_INTRIN_FUN_TABLE
};   /* }}} */

static const char* usage_strings[] = { /* {{{ */
   "int = gtk_databox_get_enable_zoom(GtkWidget)",
   "gtk_databox_grid_set_vlines(GtkOpaque,int)",
   "GtkOpaque = gtk_databox_cross_simple_new(struct,uint)",
   "GtkWidget = gtk_databox_get_ruler_x(GtkWidget)",
   "GtkAdjustment = gtk_databox_get_adjustment_y(GtkWidget)",
   "(GtkWidget, GtkWidget) = gtk_databox_create_box_with_scrollbars_and_rulers(int,int,int,int)",
   "gtk_databox_set_adjustment_x(GtkWidget,GtkAdjustment)",
   "int = gtk_databox_get_enable_selection(GtkWidget)",
   "GtkAdjustment = gtk_databox_get_adjustment_x(GtkWidget)",
   "gtk_databox_set_enable_zoom(GtkWidget,int)",
   "gtk_databox_set_ruler_y(GtkWidget,GtkWidget)",
   "gtk_databox_set_adjustment_y(GtkWidget,GtkAdjustment)",
   "gtk_databox_zoom_out(GtkWidget)",
   "GtkWidget = gtk_databox_get_ruler_y(GtkWidget)",
   "gtk_databox_graph_remove_all(GtkWidget)",
   "int = gtk_databox_grid_get_vlines(GtkOpaque)",
   "gtk_databox_set_ruler_x(GtkWidget,GtkWidget)",
   "GtkOpaque = gtk_databox_grid_new(int,int,struct,uint)",
   "gtk_databox_markers_set_label(GtkOpaque,uint,int,string,int)",
   "gtk_databox_zoom_to_selection(GtkWidget)",
   "(float, float, float, float) = gtk_databox_get_visible_limits(GtkWidget)",
   "gtk_databox_set_total_limits(GtkWidget, float, float, float, float)",
   "gtk_databox_set_enable_selection(GtkWidget,int)",
   "GtkOpaque = gtk_databox_lines_new(float [],float [],struct,uint)",
   "gtk_databox_set_visible_limits(GtkWidget, float, float, float, float)",
   "int = gtk_databox_graph_add(GtkWidget,GtkOpaque)",
   "gtk_databox_grid_set_hlines(GtkOpaque,int)",
   "gtk_databox_markers_set_position(GtkOpaque,uint,int)",
   "(float, float, float, float) = gtk_databox_calculate_extrema(GtkWidget)",
   "gtk_databox_graph_remove(GtkWidget,GtkOpaque)",
   "GtkOpaque = gtk_databox_markers_new(float [],float [],struct,uint,int)",
   "(float, float, float, float) = gtk_databox_get_total_limits(GtkWidget,float [],float [])",
   "GtkWidget = gtk_databox_new()",
   "int = gtk_databox_grid_get_hlines(GtkOpaque)",
   "gtk_databox_zoom_home(GtkWidget)",
   "",
   "GtkOpaque = gtk_databox_bars_new(float [],float [],struct,uint)",
   "GtkOpaque = gtk_databox_points_new(float [],float [],struct,int)",
   "gtk_databox_auto_rescale(GtkOpaque,float)",
   "gtk_databox_graph_set_hide(GtkOpaque,int)",
   "int = gtk_databox_graph_get_hide(GtkOpaque)",
   "int = gtk_databox_graph_get_size(GtkOpaque)",
   "struct = gtk_databox_graph_get_color(GtkOpaque)",
   "(float, float, float, float) = gtk_databox_graph_calculate_extrema(GtkOpaque)",
   "gtk_databox_graph_set_color(GtkOpaque,struct)",
   "gtk_databox_graph_set_size(GtkOpaque,int)",
   "gtk_databox_set_scale_type_x(GtkWidget, uint)",
   "gtk_databox_set_scale_type_y(GtkWidget, uint)",
   "uint = gtk_databox_get_scale_type_x(GtkWidget)",
   "uint = gtk_databox_get_scale_type_y(GtkWidget)",
   "GtkWidget = gtk_databox_ruler_new()",
   "gtk_databox_ruler_set_range(GtkWidget, float, float, float)",
   "gtk_databox_ruler_set_max_length(GtkWidget, uint)",
   "gtk_databox_ruler_set_scale_type(GtkWidget, uint)",
   "(float, float, float) = gtk_databox_ruler_get_range(GtkWidget)",
   "uint = gtk_databox_ruler_get_max_length(GtkWidget)",
   "uint = gtk_databox_ruler_get_scale_type(GtkWidget)",
   "float = gtk_databox_pixel_to_value_x (GtkWidget, int)",
   "float = gtk_databox_pixel_to_value_y (GtkWidget, int)",
   "int = gtk_databox_value_to_pixel_x (GtkWidget, float)",
   "int = gtk_databox_value_to_pixel_y (GtkWidget, float)",
   "sl_gtk_databox_graph_set_gc (GtkWidget, GdkGC)",
   "GdkGC = sl_gtk_databox_graph_get_gc (GtkWidget)",
NULL
}; /* }}} */

static void Slirp_usage(int i, int last, int flags) /* {{{ */
{
   char *indent;
   int npop = SLstack_depth();
   if (npop > SLang_Num_Function_Args) npop = SLang_Num_Function_Args;
   SLdo_pop_n(npop);
   if (last == i)
	indent = (char*)"Usage:  ";
   else {
	indent = (char*)"\t";
	SLang_verror(SLEU, (char*)"Usage: one of");
   }
   do
	SLang_verror(SLEU, (char*)"%s%s", indent, usage_strings[i++]);
   while (i < last);
   if (flags & 0x2)
	SLang_verror(SLEU,
		(char*)"\tThis function has been vectorized and parallelized.");
   else if (flags & 0x1)
	SLang_verror(SLEU, (char*)"\tThis function has been vectorized.");
} /* }}} */

static SLang_IConstant_Type gtkdatabox_IConsts [] =	/* {{{ */
{
   MAKE_ICONSTANT((char*)"GTK_DATABOX_MARKERS_SOLID_LINE",GTK_DATABOX_MARKERS_SOLID_LINE),
   MAKE_ICONSTANT((char*)"GTK_DATABOX_MARKERS_NONE",GTK_DATABOX_MARKERS_NONE),
   MAKE_ICONSTANT((char*)"GTK_DATABOX_MARKERS_TEXT_NE",GTK_DATABOX_MARKERS_TEXT_NE),
   MAKE_ICONSTANT((char*)"GTK_DATABOX_MARKERS_TEXT_SW",GTK_DATABOX_MARKERS_TEXT_SW),
   MAKE_ICONSTANT((char*)"GTK_DATABOX_MARKERS_TEXT_SE",GTK_DATABOX_MARKERS_TEXT_SE),
   MAKE_ICONSTANT((char*)"GTK_DATABOX_MARKERS_TRIANGLE",GTK_DATABOX_MARKERS_TRIANGLE),
   MAKE_ICONSTANT((char*)"GTK_DATABOX_MARKERS_DASHED_LINE",GTK_DATABOX_MARKERS_DASHED_LINE),
   MAKE_ICONSTANT((char*)"GTK_DATABOX_MARKERS_TEXT_N",GTK_DATABOX_MARKERS_TEXT_N),
   MAKE_ICONSTANT((char*)"GTK_DATABOX_MARKERS_C",GTK_DATABOX_MARKERS_C),
   MAKE_ICONSTANT((char*)"GTK_DATABOX_MARKERS_E",GTK_DATABOX_MARKERS_E),
   MAKE_ICONSTANT((char*)"GTK_DATABOX_MARKERS_N",GTK_DATABOX_MARKERS_N),
   MAKE_ICONSTANT((char*)"GTK_DATABOX_MARKERS_TEXT_CENTER",GTK_DATABOX_MARKERS_TEXT_CENTER),
   MAKE_ICONSTANT((char*)"GTK_DATABOX_MARKERS_TEXT_E",GTK_DATABOX_MARKERS_TEXT_E),
   MAKE_ICONSTANT((char*)"GTK_DATABOX_MARKERS_TEXT_NW",GTK_DATABOX_MARKERS_TEXT_NW),
   MAKE_ICONSTANT((char*)"GTK_DATABOX_MARKERS_S",GTK_DATABOX_MARKERS_S),
   MAKE_ICONSTANT((char*)"GTK_DATABOX_MARKERS_TEXT_S",GTK_DATABOX_MARKERS_TEXT_S),
   MAKE_ICONSTANT((char*)"GTK_DATABOX_MARKERS_TEXT_W",GTK_DATABOX_MARKERS_TEXT_W),
   MAKE_ICONSTANT((char*)"GTK_DATABOX_MARKERS_W",GTK_DATABOX_MARKERS_W),
   MAKE_ICONSTANT((char*)"GTK_DATABOX_SCALE_LINEAR",GTK_DATABOX_SCALE_LINEAR),
   MAKE_ICONSTANT((char*)"GTK_DATABOX_SCALE_LOG",GTK_DATABOX_SCALE_LOG),
   SLANG_END_ICONST_TABLE
};	/* }}} */

#define   SLIRP_VERSION	10903
SLANG_MODULE(gtkdatabox);
int init_gtkdatabox_module_ns(char *ns_name)	/* {{{ */
{
   SLang_NameSpace_Type *ns = NULL;

   if (slang_abi_mismatch()) return -1;
   if (ns_name != NULL) {
	ns = SLns_create_namespace (ns_name);
	if (ns == NULL) return -1;
   }


   if (allocate_reserved_opaque_types() == -1) return -1;
   /* if (allocate_gtkdatabox_opaque_types() == -1) return -1; */

   if ( -1 == SLns_add_iconstant_table(ns,gtkdatabox_IConsts,NULL) ||
	-1 == SLns_add_intrin_fun_table (ns,gtkdatabox_Funcs,(char*)"__gtkdatabox__"))
		return -1;

   return 0;
} /* }}} */
